/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#include <rpc/rpc.h>
#include "../generated/mcrpc.h"
#include <sys/time.h>

bool_t
xdr_workstationprognum(xdrs, objp)
	XDR *xdrs;
	workstationprognum *objp;
{
	if (!xdr_int(xdrs, &objp->processid)) {
		return (FALSE);
	}
	if (!xdr_int(xdrs, &objp->processno)) {
		return (FALSE);
	}
	if (!xdr_u_long(xdrs, &objp->prognum)) {
		return (FALSE);
	}
	if (!xdr_string(xdrs, &objp->modula_p_wsname, ~0)) {
		return (FALSE);
	}
	if (!xdr_string(xdrs, &objp->commid, ~0)) {
		return (FALSE);
	}
	if (!xdr_string(xdrs, &objp->real_wsname, ~0)) {
		return (FALSE);
	}
	return (TRUE);
}
