/*
 * rpcremot.h
 */


/*
 * extract of the generated h-file of mcrpccom.x
 */
 
#define COMMUNICATION ((u_long)prognum)
#define V1 ((u_long)1)
#define COMMUNICATION_PROCEDURE ((u_long)1)
extern void *communication_procedure_1();
#define RPC_SERVICE_STOP ((u_long)99)
extern void *rpc_service_stop_1();

/*
 * manually defined
 */

extern void	communication_1();
