/*
 *	mcremote.c
 */

#include <signal.h>
#include "mcfktns.h"


/*
 * Signal-Steuerung
 */


void
sig_set()
{
	int	i;

	for (i = 1; i <= NSIG; i++) {
		signal (i, sig_save [i]);
	}

	return;
}



/*
 * Program-Abort
 */

void
modula_p_abort()
{
	rpc_service_stop_client_site (NULL);
}
