/*
 *	expr.h
 */

extern YYSTYPE	eval_constexpr ();
extern YYSTYPE	eval_const_stdfktn ();
extern YYSTYPE	eval_expr ();

extern char	*ex_pos();
#define clr_expr_pos()	ex_pos (0)
#define put_expr_pos()	ex_pos (1)
#define get_expr_pos()	ex_pos (2)
#define show_expr_pos()	ex_pos (3)

