/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#include <rpc/types.h>

#include <sys/time.h>

struct workstationprognum {
	int processid;
	int processno;
	u_long prognum;
	char *modula_p_wsname;
	char *commid;
	char *real_wsname;
};
typedef struct workstationprognum workstationprognum;
bool_t xdr_workstationprognum();

#define INITIALIZATION_SERVER ((u_long)prognum_of_initialization_server)
#define V1 ((u_long)1)
#define PROCMOD_REGISTER ((u_long)1)
extern void *procmod_register_1();
#define PROCMOD_UNREGISTER ((u_long)2)
extern void *procmod_unregister_1();
#define PROCMOD_CHECK_ACTIVE ((u_long)3)
extern int *procmod_check_active_1();
#define RPC_SERVICE_STOP ((u_long)99)
extern void *rpc_service_stop_1();
