
typedef union  {
	struct s {
		Flag	flag;

		union art {
			SymbTblEntry	*stbltyp;	/* pointer to symbtblentry of type */
			BlockType	blcktyp;
			OpArt	oa;
			DefArt	da;
		} art;

		union val {
			long	lv;	/* long value */
			double	dv;	/* double value */
			char	*sv;	/* string (pointer) value */
			char	ov[16];	/* operator (string) value */
			SymbTblEntry	*pstbl;	/* pointer to symbtblentry */
		} val;
	} s;
} YYSTYPE;
extern YYSTYPE yylval;
# define AND 257
# define ARRAY 258
# define BEG 259
# define BY 260
# define CASE 261
# define CALL 262
# define COMMUNICATION 263
# define CONST 264
# define DISABLE 265
# define DO 266
# define ELSE 267
# define ELSIF 268
# define ENABLE 269
# define END 270
# define ENTRY 271
# define EVENT 272
# define EXCEPTION 273
# define EXCEPTNO 274
# define EXIT 275
# define EXPORT 276
# define EXTERNAL 277
# define FOR 278
# define FORWARD 279
# define HALT 280
# define HIGH 281
# define IF 282
# define IMPLEMENTATION 283
# define IMPORT 284
# define INITCOM 285
# define INTERRUPT 286
# define LOOP 287
# define LOWLEVEL 288
# define MODULE 289
# define MONITOR 290
# define NOT 291
# define OF 292
# define OR 293
# define PROCEDURE 294
# define PROCESS 295
# define PROCESSID 296
# define PROCESSNO 297
# define PROCESSOR 298
# define RAISE 299
# define RECORD 300
# define REPEAT 301
# define RESTART 302
# define RESUME 303
# define RETURN 304
# define SIGNAL 305
# define START 306
# define STATUS 307
# define THEN 308
# define TO 309
# define TYPE 310
# define UNTIL 311
# define VAL 312
# define VAR 313
# define WAIT 314
# define WHILE 315
# define WITH 316
# define XOR 317
# define BOOLEAN 318
# define COMPUTER 319
# define CONDITION 320
# define POINTER 321
# define SEMAPHORE 322
# define SET 323
# define ASSIGNMENTOP 324
# define RELOP 325
# define MULOP 326
# define PLUSMINUS 327
# define TWOSINGLEDOTS 328
# define INTNUMBER 329
# define REALNUMBER 330
# define STRING 331
# define IDENT 332
# define NEWORDISPOSE 333
# define PORV 334
# define STDFKTN 335
