/*
 *	lex_yy.h
 */

extern char	yytext[];			/* [YYLMAX] */
extern int	yylineno;
extern FILE	*yyin;
extern char	*yysptr;
extern char	yysbuf[];			/* [YYLMAX] */

extern char	id_name[];			/* [YYLMAX + 5] */
extern char	id_name_fuer_fehlertext[];	/* [YYLMAX] */
/*	extern char	string_inhalt[];	kann ber yylval.s.val.sv zugegriffen werden */	/* [YYLMAX] */
extern int	max_token_length;		/* = YYLMAX */

extern void	lex_unput_char();		/* unputs a character to the input stream */

