Fatal: Internal errors (Open Errors):
=====================================

mc: Cannot open output file '%s'
mc: Cannot open temporary file '!premod!.tmp'
mc: Cannot open initialization file '!premod!.ini'
mc: Cannot open output file '%s'
yyparse (commbody): Cannot open output file '%s'
yyparse (commbody): Cannot open output file '%s' to append
comm_out_rpc_file: Cannot open file '%s' to read


Fatal: Internal errors:
=======================

main: getcwd of current directory failed
main: getpwuid failed
main: getpwnam failed
main: chdir to home directory failed
main: chdir to current directory failed
main: getcwd of home directory failed
strsave: No more memory for String '%s' to save
newblocklist: No more memory for new BlockList-Entry
newsymbtbl: No more memory for Symboltable-Entries
entryinactualblock: 'entry' or 'blocklist' has Value NULL
deleteentry: Entry cannot be deleted from hashlist [%d]
deleteentry: Entry cannot be deleted from its blocklist
typeadd: No more memory for Types
setactualblocklist: Block not found
yyparse (importlistid): No more memory for ImportSave
yyparse (importlistid): No more memory for yysbuf to save
add_filename: No more memory for FileNames
c_saved_out: Cannot open temporary file '!premod!.tmp'
c_saved_kill: Cannot open temporary file '!premod!.tmp'
c_init_out: Cannot open initialization file '!premod!.ini'
add_stbl_list: No more memory for STblList
yyparse (idlistident): unknown idlisttype %d
message: exit within message
loop_stack: Unknown Parameter %s
yyparse (constantexpr): Type of Constant-Identifier '%s' invalid
c_exbuf: Expression buffer length exceeded! Split Expression
c_exbuf: Filenames too long for automatic C-Compilation
c_exbuf: Too many Object Files for automatic Link
mc_link: Unable to link Object Code of File '%s'
deleteactualblock: Inner Block is of no Procedure-Type
deleteactualblock: Cannot delete symboltable-entry
insert_exbuf: Expression-Buffer-Pointer out of Position
yyparse (statement): No more memory for INITCOMLIST
yyparse (statement): No more memory for COMPILELIST
yyparse (statement): Blocklist of Monitor not found
yyparse (designator): Invalid Type Conversion Function
yyparse (designator): Unknown Conversion Function
save_stbl_list: Parameter '%d' invalid
save_stbl_list (1): No more memory for STblListSave
save_stbl_list (-1): Stbllist is not empty
save_stbl_list (-1 or -2): Stbllistsave is empty
yyparse (recordtype): No more memory for new Type
yyparse (constorcomputer): No more memory for WorkstationList
yyparse (constorcomputer): No more memory for Workstation


Fatal errors:
=============

no_recovery: Error Recovery impossible at Token '%s' because of a previously detected error
yyparse (moduletypes): No Error-Recovery possible
yyparse (type): Integer-Type not found
yyparse (moduledecl): Module-Identifier '%s' already declared
lookforgentype: INTEGER-Type not found
yylex: Unexpected End of File
yylex (STRING): Char-Type not found
yylex (STRING): STRING-Type not found
yylex (REALNUMBER): SHORTREAL-Type not found
yylex (DECIMAL): SHORTREAL-Type not found
unsigned_const_stbltyp: Byte-Type not found
unsigned_const_stbltyp: Shortcard-Type not found
unsigned_const_stbltyp: Card-Type not found
unsigned_const_stbltyp: Longcard-Type not found
unsigned_const_stbltyp: Positive Shortint-Type not found
unsigned_const_stbltyp: Positive Int-Type not found
unsigned_const_stbltyp: Longint-Type not found
negative_const_stbltyp: Negative Type of Longcard doesn't exist
negative_const_stbltyp: Negative Type of Card doesn't exist
negative_const_stbltyp: Type '%s' unknown in this procedure
negative_const_stbltyp: Shortint-Type not found
negative_const_stbltyp: Int-Type not found
negative_const_stbltyp: Longint-Type not found
loop_stack: Maximum Number of inner Loops (15) exceeded
mc: Cannot open input file '%s'
yyparse (specialtype): Semaphore-Type not found
yyparse (specialtype): Condition-Type not found
eval_constexpr: Boolean-Type not found
eval_const_stdfktn: SHORTREAL-Type not found
eval_const_stdfktn: CHAR-Type not found
eval_const_stdfktn: Invalid Function '%s'
eval_expr: Boolean-Type not found
yyparse (type): Boolean-Type not found
yyparse (msklenconst): Boolean-Type not found
yyparse (subr): Array too complex; please split into more types
ex_pos: Expression too complex. Expression-Buffer overflow at Token '%s'
ex_pos: Illegal Switch '%d'
set_openarray_of_stbl_list: POSITIVEINT-Type not found
initprecompiler: POSITIVEINT-Type not found
yyparse (constorcomputer): COMPUTER-Type not found
yyparse (designator): BYTE-Type not found
yyparse (designator): CHAR-Type not found
yyparse (designator): SHORTREAL-Type not found
yyparse (designator): BYTE-Type not found for STATUS
yyparse (designator): BYTE-Type not found for HIGH
yyparse (designator): INTEGER-Type not found for PROCESSID
yyparse (designator): INTEGER-Type not found for PROCESSNO
yyparse (designator): SHORTCARD-Type not found for EXCEPTNO
yyparse (block): Semaphore-Type not found
define_record: Record has too much elements or too long fieldnames; please split into more (sub-)types
out_array_of_cond_or_sem: Unknown Type (semaphore or condition expected)


Compilation errors:
===================

remote_compile: Error at Remote Compilation, Return Code '%d'
yyerror: %s at token '%.80s'
yyerror: %s
lookforgentype: Type-Entry of Symbtbl-Entry '%s' not found; INTEGER-Type assumed
yyparse (importlistid): IMPORT-File '%s' not found
yyparse (ident): Identifier '%s' not declared, assumed as external integer
yyparse (processormodule): Only one PROCESSOR MODULE allowed
yyparse (modulebody): Module-Identifier '%s' not declared
yyparse (procedurebody): Procedure-Identifier '%s' not declared
yyparse (procedurebody): Procedure '%s' is not the current declared Procedure
yyparse (restofeventhead): Interrupt-Identifier '%s' not declared or event defined twice
yyparse (eventbody): Event-Identifier '%s' not declared
yyparse (eventbody): Event '%s' is not the current declared Event
yyparse (processbody): Process-Identifier '%s' not declared
yyparse (processbody): Process '%s' is not the current declared Process
yyparse (commbody): rpcgen failed
yyparse (commbody): Communication-Identifier '%s' not declared
yyparse (commbody): Communication '%s' is not the current declared Communication
yyparse (monbody): Monitor-Identifier '%s' not declared
yyparse (monbody): Monitor '%s' is not the current declared Monitor
yyparse (entrybody): Entry-Identifier '%s' not declared
yyparse (entrybody): Entry '%s' is not the current declared Entry
yyparse (ident): 'iduse' not set
yyparse (type): Type-Identifier '%s' is not declared. Assumed as integer
symbtbladd: Identifier '%s' is a reserved name
symbtbladd: Identifier '%s' already declared
yylex: HEXADECIMAL-Constant '%s' too big (> 4 Byte)
yylex: BINARY-Constant '%s' too big (> 32 Bit)
yyparse (_byexpr_): Increment Expression is not valid
mintype: Type ('%s') not compatible with Type ('%s')
maxtype: Type ('%s') not compatible with Type ('%s')
yyparse (whilestat): Expression is not of type BOOLEAN
yyparse (qualident): Identifier '%s' not declared
yyparse (blockend): Unmatched END in block
yyparse (constorcomputer): Constant-Type not recognized
yyparse (constorcomputer): Declarations of Type COMPUTER only at the Constant Declarationpart of the Processor-Module allowed
yyparse (constorcomputer): Procedure COMPUTER only at the Constant Declarationpart of the Processor-Module allowed
yyparse (constorcomputer): Workstation-Class must be a String
yyparse (constorcomputer): Workstation-Name must be a String
yyparse (constorcomputer): Workstation-Name already defined! Use '%s = %s'
yyparse (exthenstat): Expression is not of type BOOLEAN
yyparse (repexpr): Expression is not of type BOOLEAN
yyparse (_address_): Address only in Lowlevel-Modules allowed
yyparse (_address_): Variable-Identifier '%s' not declared
yyparse (_address_): Variable-Identifier '%s' must be fully declared before address-reference
yyparse (_var_): Call by Reference not allowed for Processes
yyparse (msklenconst): BOOLEAN with Bitmask only in Lowlevel-Modules allowed
yyparse (specialtype): SEMAPHORE only in Lowlevel-Modules allowed
yyparse (specialtype): SEMAPHORE-Type as Component of Records or Pointers not allowed
yyparse (specialtype): SEMAPHORE-Type only in Variable-Declaration allowed
yyparse (specialtype): CONDITION only in Monitors allowed
yyparse (specialtype): CONDITION-Type as Component of Records or Pointers not allowed
yyparse (specialtype): CONDITION-Type only in Variable-Declaration allowed
yyparse (specialtype): BOOLEAN-Type with Bitmask only in Variable-Declaration allowed
yyparse (totypeornot): Untyped POINTER only in Lowlevel-Modules allowed
yyparse (constantexpr): Only boolean Values allowed with Operator 'NOT'
yyparse (constantexpr): Only numeric Values allowed with Operator 'Unary %s'
yyparse (constantexpr): Constant-Identifier '%s' not declared
yyparse (constantexpr): Only Standard Functions FLOAT, TRUNC, ORD, or CHR allowed at constant declaration part
yyparse (modulebody): Module-Identifier '%s' not the same as in declaration part
yyparse (case): Type in CASE-List incompatible with Type in CASE-Expression
yyparse (caselabellist): Incompatible Types in CASE-List
yyparse (_toconstantexpr_): Incompatible Types at '..'
yyparse (_toconstantexpr_): Constant after '..' must be less than constant before
yyparse (_toconstantexpr_): CASE-Label-Expression must be of an Integer- or Cardinal-Type
eval_constexpr: Operands and/or Operandtype incompatible in Constant Expression
eval_constexpr: Type overflow at Operator '%s'
eval_constexpr: Boolean Expression impossible with XOR
eval_constexpr: Integer Expression with AND, OR, XOR only in Lowlevel-Modules allowed
eval_const_stdfktn: Invalid argument type at function FLOAT
eval_const_stdfktn: Invalid argument type at function TRUNC
eval_const_stdfktn: Invalid argument type at function ORD
eval_const_stdfktn: Invalid argument type at function CHR
yyparse (expression): Only boolean Values allowed with Operator 'NOT'
yyparse (expression): Only numeric Values allowed with Operator 'Unary %s'
eval_expr: Boolean Expression impossible with XOR
eval_expr: Integer Expression with AND, OR, XOR only in Lowlevel-Modules allowed
eval_expr: Operands and/or Operandtype incompatible in Expression
eval_expr: Type overflow at Operator '%s'
ex_pos: Expression-Buffer underflow at Token '%s'. Cause is an earlier Error
yyparse (casestat): CASE-Expression must be of an Char-, Enumeration-, Subrange-, Integer- or Cardinal-Type
yyparse (caselabel): CASE-Label-Expression must be of an Char-, Enumeration-, Integer- or Cardinal-Type
yyparse (_toconstantexpr_): CASE-Label-Expression must be of an Char-, Enumeration-, Integer- or Cardinal-Type
yyparse (forstat): FOR-Variable-Identifier is of an invalid Type
yyparse (forstat): Incompatible Types at TO-Expression
yyparse (_byexpr_): BY-Constant-Expression only with FOR-Variables of Integer- or Cardinal-Type
yyparse (intdecllistid): Interrupt-Number must be of Type BYTE
search_for_undeclared_events: Event to corresponding Interrupt '%s' not declared
yyparse (_address_): Constant-Expression of invalid Type
yyparse (_address_): Constant-Expression too big (max. 2 Byte)
yyparse (subrange): Subrange of invalid Type
yyparse (subrange): Value of the first Subrange-Parameter must be less than Value of the seconde one
yyparse (desex): Not of an Array-Type
yyparse (desex): Array-Index of invalid Type
yyparse (desex): Array-Index '%ld' out of range ('%ld', '%ld')
yyparse (countornot): START or INITCOM only with second argument of INTEGER- or CARDINAL-Type allowed
yyparse (assignment): Assignment to Open-Array impossible
yyparse (assignment): Variable-Identifier left of ':=' is used in outer FOR-Statement
yyparse (assignment): Incompatible Types at Subrange-Assignment
yyparse (assignment): Incompatible Types at Pointer-Assignment
yyparse (assignment): Incompatible Types at Assignment
yyparse (assignment): Incompatible String-Types at Assignment
yyparse (assignment): Identifier left of ':=' must be lvalue (no Constant or Procedurecall)
yyparse (designatorexpr): Array-Identifier is invalid
yyparse (designatorexpr): Record-Identifier is invalid
yyparse (designatorexpr): Not of a Record-Type
yyparse (designatorexpr): Record-Field-Identifier '%s' not declared
yyparse (designatorexpr): Pointer-Identifier is invalid
yyparse (designatorexpr): Not of a Pointer-Type
yyparse (designatorexpr): Pointer-Operator '^' not allowed with an untyped Pointer
yyparse (restofprocedhd): Procedure already defined
yyparse (semicolextforw): Head of Procedure must be separated from 'EXTERNAL' with a semicolon
yyparse (semicolextforw): Head of Procedure must be separated from 'FORWARD' with a semicolon
yyparse (idlistident): Parameter-Count differs from earlier Declaration
yyparse (idlistident): Parameter-Identifier '%s' differs from earlier Declaration
yyparse (_formalparams_): Parameter-Count differs from earlier Declaration
yyparse (.designatorexpr.): Count of Actual Parameters differs from Count of defined Formal-Parameters
yyparse (actualparams): Procedure-Identifier is invalid
yyparse (actualparams): Monitor-Identifier cannot be used like Procedure-Identifier
yyparse (actualparams): Count of Actual Parameters is less than Count of defined Formal-Parameters
yyparse (actexp): Count of Actual Parameters is higher than Count of defined Formal-Parameters
yyparse (actexp): Invalid Parameter-Type
yyparse (actexp): Call-By-Reference-Parameter must not be a global declared variable of a lowlevel module
yyparse (actexp): Parameter must be an lvalue
yyparse (expression): Entry-Call as Expression only at an Assignment-Statement possible
yyparse (procedurecall): Procedure-Call as single Statement only with Procedure-Type void allowed
yyparse (commcall): Computer-Identifier '%s' not found
yyparse (commcall): Identifier '%s' is not of type COMPUTER
yyparse (commcall): Communication-Call as single Statement only with Entry-Type void allowed
yyparse (entrycalloldver): Entry-Call as single Statement only with Entry-Type void allowed
yyparse (entrycall): Entry-Call as single Statement only with Entry-Type void allowed
yyparse (entrycall): Monitor-Identifier invalid
yyparse (entrycall): Entry belongs to Monitor '%s' instead of Monitor '%s'
yyparse (block): Empty Block not allowed for Communications (at least keyword BEGIN required)
yyparse (block): Empty Block only with Procedure-Type void allowed
yyparse (returnstat): RETURN-Statement only in COMMUNICATION, ENTRY or PROCEDURE allowed
yyparse (returnstat): Incompatible Type of RETURN-Expression
yyparse (returnstat): RETURN-Expression missing
yyparse (actexp): ARRAY-Type required
loop_stack (0): EXIT only within a Loop-Statement allowed
yyparse (statement): Procedure WAIT only in Entry-Block allowed
yyparse (statement): WAIT-Parameter is not of type CONDITION
yyparse (statement): Procedures P and V only in Lowlevel-Modules allowed
yyparse (statement): P- or V-Parameter is not of type SEMAPHORE
yyparse (statement): Procedure SIGNAL only in Entry-Block allowed
yyparse (statement): SIGNAL-Parameter is not of type CONDITION
yyparse (statement): RAISE only with argument of SHORTCARD-Type allowed
yyparse (statement): RAISE is forbidden within an Exception
yyparse (statement): RESUME is forbidden outside of an Exception
yyparse (statement): Procedure INITCOM only in the Initialization-Block of the Processor-Module allowed
yyparse (statement): First Parameter of INITCOM is no Communication Identifier
yyparse (statement): Second Parameter of INITCOM is no Constant Identifier
yyparse (statement): Second Parameter of INITCOM is not of type COMPUTER
yyparse (statement): INITCOM (%s, %s) cannot be used more than once
yyparse (statement): Name '%s_%s_%s' too long for a communication server program
yyparse (statement): RESTART is forbidden outside of an Exception
yyparse (designator): Identifier '%s' doesn't exist
yyparse (designator): Identifier '%s' is of Type SEMAPHORE or CONDITION, Assignments are not allowed
yyparse (designator): Procedure STATUS only in Entry-Block allowed
yyparse (designator): STATUS-Parameter is not of type CONDITION
yyparse (designator): HIGH-Parameter is not an Open Array Parameter
yyparse (designator): Parameter '%s' of Procedure HIGH is not an Open Array Parameter
yyparse (block): Procedures returning a Value must have a RETURN-Statement
yyparse (designator): INTEGER- or CARDINAL-Argument expected at Type Conversion Function
yyparse (designator): REAL-Argument expected at Type Conversion Function
yyparse (designator): Invalid Argument Type at Type Conversion Function
yyparse (designator): Float only with INTEGER- or CARDINAL-Argument allowed
yyparse (designator): Trunc only with REAL-Argument allowed
yyparse (designator): Ord only with Enumeration-Type-, BOOLEAN- or CHAR-Argument allowed
yyparse (designator): Chr only with BYTE-Argument allowed
yyparse (designator): First VAL-Parameter is no Enumeration-Type-Identifier
yyparse (designator): Second VAL-Parameter is not of Type BYTE
yyparse (designator): Dec and Inc only with Enumeration-Type-, CHAR-, Integer od Cardinal-Argument allowed
yyparse (designator): Dec- and Inc-Argument must be lvalue (no Constant or Procedurecall)
yyparse (designator): Argument is used in outer FOR-Statement
yyparse (designator): New and Dispose only with Pointer-Type-Argument allowed
yyparse (designator): New and Dispose not with untyped Pointer-Argument allowed
yyparse (designator): EXCEPTNO is forbidden outside of an Exception
yyparse (_seminit_): Only Cardinal Constants allowed as SEMAPHORE-Init-Value
check_type_of_stbl_list: Parameter was already declared as Open Array
check_type_of_stbl_list: Parameter-Type differs from earlier Declaration
check_type_of_stbl_list: Parameter was already declared as Call-By-Reference
check_type_of_stbl_list: Parameter was already declared as Call-By-Value
yyparse (_oftypeident_): Type-Identifier not allowed for Processes
yyparse (_oftypeident_): Procedure-Type differs from earlier Declaration
yyparse (suborid): Subrange-Identifier '%s' not declared
yyparse (suborid): Subrange-Identifier '%s' not a subrange or enumeration
yyparse (disenable): Within a DISABLE-ENABLE-block only assignments are allowed
yyparse (arraytype): ARRAY OF only in Parameter-Section allowed (not recursive); ignored
yyparse (restofprocesshd): Process already defined
yyparse (restofmonhead): Monitor already defined
yyparse (restofentryhead): Entry already defined
deleteactualblock: Predeclared Identifier '%s' of 'Procedure-Kind' is not fully declared
local_communication_out: OPEN ARRAYS not allowed within Communications
local_communication_out: OPEN ARRAYS as Call-By-Reference-Parameter not allowed within Communications


Warnings:
=========

yyerror (yyparse (module)): '%s' received after end of Module; rest of file ignored.
add_filename: Filename '%s' already exists in List
enumeration: Count of Elements '%d' exceeds 256
yylex: DECIMAL-Constant '%s' too big; treated as REALNUMBER
yylex: Token '%s' is a reserved word. It must be written uppercase
yyparse (commaornot): Token ',' expected at List Separation
yyparse (noprocedure): Token PROCEDURE not allowed at end of Procedure
yyparse (noevent): Token EVENT not allowed at end of Event
yyparse (noentry): Token ENTRY not allowed at end of Entry
yyparse (commornot): Token COMMUNICATION is missing
yyparse (processorornot): Token PROCESSOR is missing
yyparse (lowlevelornot): Token LOWLEVEL is missing
yyparse (moduleornot): Token MODULE is missing
yyparse (processornot): Token PROCESS is missing
yyparse (exceptionornot): Token EXCEPTION is missing
yyparse (monitorornot): Token MONITOR is missing
yyparse (block): Token BEGIN is missing

